# frozen_string_literal: true

#
# Copyright (c) 2016-2021 Yegor Bugayenko
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the 'Software'), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

require 'xembly'
require 'nokogiri'

module Xembly
  # ADDIF directive
  class AddIf
    # Ctor.
    # +name+:: Node name to add
    def initialize(name)
      @name = name
    end

    def exec(dom, cursor)
      after = []
      cursor.each do |node|
        if node.element_children.none? { |e| e.name == @name }
          child = Nokogiri::XML::Node.new(@name, dom)
          node.add_child(child)
          after.push(child)
          Xembly.log.info "node \"#{@name}\" added to \"#{node.name}\""
        else
          after.push(node)
        end
      end
      after
    end
  end
end
